/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: game_viewer.cpp,v $
Revision 1.28  2007/04/26 17:18:33  ilariacas
release finale

Revision 1.27  2007/04/25 22:35:58  ilariacas
minor modif

Revision 1.26  2007/04/25 14:19:15  ilariacas
dynamic depth of field

Revision 1.25  2007/04/19 22:02:51  ilariacas
working on depth of field

Revision 1.24  2007/04/16 23:13:51  ilariacas
just code cleaning

Revision 1.23  2007/04/16 10:43:41  ilariacas
dynamic cube mapping is working

Revision 1.22  2007/04/14 17:42:54  ilariacas
lens flare - missing correct focus_plane

Revision 1.21  2007/04/13 14:04:17  ilariacas
better focus_plane (?right?)

Revision 1.20  2007/04/13 11:45:33  ilariacas
GL_ACCUM enabled - missing focus_plane

Revision 1.19  2007/04/12 22:56:12  ilariacas
depth of field draft (not working)

Revision 1.18  2007/04/12 22:27:35  ilariacas
depth of field draft (not working)

Revision 1.17  2007/04/10 21:37:30  ilariacas
generic modif

Revision 1.16  2007/04/09 22:00:39  ilariacas
building textures and display list

Revision 1.15  2007/04/07 17:07:34  ilariacas
minor modif

Revision 1.14  2007/04/06 17:20:35  ilariacas
skybox modif

Revision 1.13  2007/04/05 21:52:25  ilariacas
skybox modif

Revision 1.12  2007/04/03 17:14:51  ilariacas
skybox

Revision 1.11  2007/04/02 16:40:13  ilariacas
cube mapping enabled on the car

Revision 1.10  2007/03/31 17:25:20  ilariacas
billboarding and draft of buildings

Revision 1.9  2007/03/30 17:54:58  ilariacas
first draft of billboarding

Revision 1.8  2007/03/24 13:13:59  ilariacas
*** empty log message ***

Revision 1.7  2007/03/21 15:20:18  ilariacas
load ply (OK)

Revision 1.6  2007/03/20 22:39:13  ilariacas
load ply (working - TOO SLOW)

Revision 1.5  2007/03/20 22:17:39  ilariacas
load ply (not working)

Revision 1.4  2007/03/20 20:22:05  ilariacas
road

Revision 1.3  2007/03/20 09:51:38  ilariacas
lights

Revision 1.2  2007/03/20 08:55:22  ilariacas
*** empty log message ***

Revision 1.1  2007/03/17 16:52:04  ilariacas
lookAt

Revision 1.3  2007/03/11 18:11:59  ganovelli
minor modif


****************************************************************************/

#include <SDL.h>
#include <GL/glew.h>
#include <math.h>
#include <time.h>
#include <vcg/space/point3.h>
#include <vector>
#include <GL/glut.h>
#include <GL/glu.h>
#include <stdlib.h>

#include <iostream>


//include x load modello mesh .ply
#include <vcg/simplex/vertexplus/base.h>
#include <vcg/simplex/faceplus/base.h>
#include <vcg/complex/trimesh/base.h>
#include <vcg/complex/trimesh/update/bounding.h>
#include <vcg/complex/trimesh/update/normal.h>
#include <wrap/io_trimesh/import.h>
#include <wrap/gl/trimesh.h>


#include "../client/game_client.h"
#include <wrap/gui/trackball.h>
#include "game_viewer.h"
#include "race.h"

#include "tga.h"
#include "Lens.h"

using namespace vcg;
using namespace std;

class CEdge;    // dummy prototype never used
class CFace;


class CVertex : public VertexSimp2< CVertex, CEdge, CFace, vert::Coord3f, vert::Normal3f >{};
//cosi non vedo i colori della macchina
//class CFace   : public FaceSimp2<   CVertex, CEdge, CFace, face::VertexRef, face::Normal3f > {};

//cosi vedo i colori della macchina
class CFace   : public FaceSimp2<   CVertex, CEdge, CFace, face::VertexRef, face::Normal3f , face::Color4b  > {}; 

/*the mesh is a container of vertices and a container of faces */ 
class CMesh   : public vcg::tri::TriMesh< vector<CVertex>, vector<CFace> > {};
CMesh mesh; //car mesh

vcg::GlTrimesh<CMesh> glWrap;
//vcg::Trackball track;

vcg::Point3f pov;
vcg::Point3f position;
int x,y,z;
float original_dist, dist, incr_focus_dist;

/** Client  */
extern  NvmcClient cli;

/* value of pi */
#define M_PI       3.14159265358979323846

/** viewport size */
int width,height;

/* valori per il lighting */
GLfloat mat_lig[32];

/*numero di rendering x il depth of filed*/
int DEPTH_RENDER;

typedef struct 
{
    GLfloat x, y;
} jitter_point;


/* 4 e 8 jitter points */
jitter_point j4[] =
    {
	{-0.208147,  0.353730},
	{ 0.203849, -0.353780},
	{-0.292626, -0.149945},
	{ 0.296924,  0.149994}
    };
jitter_point j8[] =
    {
	{-0.334818,  0.435331},
	{ 0.286438, -0.393495},
	{ 0.459462,  0.141540},
	{-0.414498, -0.192829},
	{-0.183790,  0.082102},
	{-0.079263, -0.317383},
	{ 0.102254,  0.299133},
	{ 0.164216, -0.054399}
    };

Flares *Fl; //lens flare

#ifdef WIN32
#define near zNear
#define far zFar
#endif

#define TexRoadFile   "../../client1/roadw2.bmp" //path texture strada
#define TexGrassFile "../../client1/erbetta.bmp" //path texture erba
#define TexTreeFile "../../client1/tree.tga" //path alberello
#define TexBuilding1File "../../client1/building.bmp" //path palazzo
#define TexBuilding2File "../../client1/palace.bmp"
#define TexBuilding3File "../../client1/apartment.bmp"

#define TexCubeXposFile "../../client1/east.bmp"//"../../client1/pos_x.bmp"
#define TexCubeXnegFile "../../client1/west.bmp"//"../../client1/neg_x.bmp"
#define TexCubeYposFile "../../client1/up.bmp"//"../../client1/pos_y.bmp"
#define TexCubeYnegFile "../../client1/down.bmp"//"../../client1/neg_y.bmp"
#define TexCubeZposFile "../../client1/north.bmp"//"../../client1/pos_z.bmp"
#define TexCubeZnegFile "../../client1/south.bmp"//"../../client1/neg_z.bmp"

#define TexFlare1File "../../client1/Flare1.tga"
#define TexFlare2File "../../client1/Flare2.tga"
#define TexFlare3File "../../client1/Flare3.tga"


#define TexRoad   0    //index texture
#define TexGrass  1
#define TexTree	  2
#define TexBuilding1 3
#define TexBuilding2 4
#define TexBuilding3 5

#define TexCubeXpos 0
#define TexCubeXneg 1
#define TexCubeYpos 2
#define TexCubeYneg 3
#define TexCubeZpos 4
#define TexCubeZneg 5

#define TexFlare1 0
#define TexFlare2 1
#define TexFlare3 2

#define NUM_TEXTURES  6           //total number of texture
#define ENV_TEXTURES 6
#define FLARE_TEXTURES 3

GLuint texName[NUM_TEXTURES]; //vettore di tutte le texture
GLuint texCube[ENV_TEXTURES];
GLuint texFlare[FLARE_TEXTURES];
// Cube map
GLuint cubeMap;

GLuint carDL, roadDL;
 
#define CAR_MODEL_PATH "../../models/cars/smart.ply" //path of car model


void initTexture(void)
{
bool wrap = true;

 // Generate textures
 glGenTextures(NUM_TEXTURES, texName);
 glGenTextures(ENV_TEXTURES, texCube);
 glGenTextures(FLARE_TEXTURES, texFlare);

 glPixelStorei(GL_UNPACK_ALIGNMENT,4);
 SDL_Surface* bmpFile;

 bmpFile = SDL_LoadBMP(TexRoadFile);

 // texture x la strada
 glBindTexture( GL_TEXTURE_2D, texName[TexRoad] );
 //glTexEnvf( GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);


 //texture x l'erba
  bmpFile = SDL_LoadBMP(TexGrassFile);
 glBindTexture( GL_TEXTURE_2D, texName[TexGrass] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

 //texture x l'albero
 tgaInfo *tga;
 tga = tgaLoad(TexTreeFile);
 glBindTexture(GL_TEXTURE_2D,texName[TexTree]);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
 glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);	
 glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tga->width, tga->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tga->imageData);

 //texture 1 x i palazzi
 bmpFile = SDL_LoadBMP(TexBuilding1File);
 glBindTexture( GL_TEXTURE_2D, texName[TexBuilding1] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

  //texture 2 x i palazzi
 bmpFile = SDL_LoadBMP(TexBuilding2File);
 glBindTexture( GL_TEXTURE_2D, texName[TexBuilding2] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_RGB, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

   //texture 3 x i palazzi
 bmpFile = SDL_LoadBMP(TexBuilding3File);
 glBindTexture( GL_TEXTURE_2D, texName[TexBuilding3] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeXposFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeXpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeXnegFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeXneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeYposFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeYpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeYnegFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeYneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeZposFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeZpos] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

  bmpFile = SDL_LoadBMP(TexCubeZnegFile);
 glBindTexture( GL_TEXTURE_2D, texCube[TexCubeZneg] );
 //glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
 glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);

 SDL_FreeSurface(bmpFile);

 //3 texture RGBA x il lens-flare
 	tga = tgaLoad(TexFlare1File);
	glBindTexture( GL_TEXTURE_2D, texFlare[TexFlare1] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tga->width, tga->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tga->imageData);

	tga = tgaLoad(TexFlare2File);
	glBindTexture( GL_TEXTURE_2D, texFlare[TexFlare2] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tga->width, tga->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tga->imageData);

	tga = tgaLoad(TexFlare3File);
	glBindTexture( GL_TEXTURE_2D, texFlare[TexFlare3] );
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MAG_FILTER	,GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D,	GL_TEXTURE_MIN_FILTER	,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tga->width, tga->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tga->imageData);

}

void init_cube_map(){
		glPixelStorei(GL_UNPACK_ALIGNMENT, 4);

    // Initialize cube map faces
    glEnable(GL_TEXTURE_CUBE_MAP_EXT);
    glGenTextures(1, &cubeMap);
    glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, cubeMap);

		////* definiamo il wrapping delle coordinate texture per ogni direzione*/
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_REPEAT);

		//*definiamo come deve essere fatto il smapling nel caso di MAGNIFICATION e di MINIFICATION */
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_NEAREST);


		SDL_Surface* bmpFile;
		bmpFile = SDL_LoadBMP(TexCubeXposFile);
		// Carica la texture per il piano ortogonale a X su x positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeXnegFile);
		// Carica la texture per il piano ortogonale a X su x negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeYposFile);
		// Carica la texture per il piano ortogonale a Y su y positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);	

		bmpFile = SDL_LoadBMP(TexCubeYnegFile);
		// Carica la texture per il piano ortogonale a Y su y negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeZposFile);
		// Carica la texture per il piano ortogonale a Z su z positivo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

		bmpFile = SDL_LoadBMP(TexCubeZnegFile);
		// Carica la texture per il piano ortogonale a Z su z negativo
		gluBuild2DMipmaps(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,GL_RGB, bmpFile->w, bmpFile->h, GL_BGR, GL_UNSIGNED_BYTE, bmpFile->pixels);
		SDL_FreeSurface(bmpFile);

    glDisable(GL_TEXTURE_CUBE_MAP_EXT);



		//*stabilisci come devono essere generate le coordinate texture */
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);

}

void enable_cube_map(){					
	///*Abilita la generazione di coordinate texture */
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	///* definisci come il risultato del texturing deve influire sul colore finale del frammento.*/
    //glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_CLAMP);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	//*abilita il cube mapping */
	glEnable(GL_TEXTURE_CUBE_MAP_EXT);   
	glEnable(GL_NORMALIZE);


}
void disable_cube_map(){
   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);
   glDisable(GL_TEXTURE_GEN_R);
   glDisable(GL_TEXTURE_CUBE_MAP_EXT);
}


void draw_skybox(float x, float y, float z, float width, float height, float length){
	glEnable (GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

	//coordinate x texturing evitando le linee dei bordi
	float zero=0.001;
	float uno=0.999;

	// Center the Skybox around the given x,y,z position
	x = x - width  / 2;
	y = y - height / 2;
	z = z - length / 2;

	glPushMatrix();
	
	// avanti
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeZneg]);
	glBegin(GL_QUADS);	
		glTexCoord2f(uno, uno); glVertex3f(x,		y,		  z+length);
		glTexCoord2f(uno, zero); glVertex3f(x,		y+height, z+length);
		glTexCoord2f(zero, zero); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(zero, uno); glVertex3f(x+width, y,		  z+length);
	glEnd();

	// dietro
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeZpos]);
	glBegin(GL_QUADS);
		glTexCoord2f(uno, uno); glVertex3f(x+width, y,		  z);
		glTexCoord2f(uno, zero); glVertex3f(x+width, y+height, z); 
		glTexCoord2f(zero, zero); glVertex3f(x,		y+height, z);
		glTexCoord2f(zero, uno); glVertex3f(x,		y,		  z);
	glEnd();

	// sinsitra
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeXneg]);
	glBegin(GL_QUADS);		
		glTexCoord2f(uno, zero); glVertex3f(x,		  y+height,	z);	
		glTexCoord2f(zero, zero); glVertex3f(x,		  y+height,	z+length); 
		glTexCoord2f(zero, uno); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(uno, uno); glVertex3f(x,		  y,		z);		
	glEnd();

	// destra
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeXpos]);
	glBegin(GL_QUADS);		
		glTexCoord2f(zero, uno); glVertex3f(x+width, y,		z);
		glTexCoord2f(uno, uno); glVertex3f(x+width, y,		z+length);
		glTexCoord2f(uno, zero); glVertex3f(x+width, y+height,	z+length); 
		glTexCoord2f(zero, zero); glVertex3f(x+width, y+height,	z);
	glEnd();

	// sopra
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeYpos]);
	glBegin(GL_QUADS);		
		glTexCoord2f(zero, zero); glVertex3f(x+width, y+height, z);
		glTexCoord2f(zero, uno); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(uno, uno); glVertex3f(x,		  y+height,	z+length);
		glTexCoord2f(uno, zero); glVertex3f(x,		  y+height,	z);
	glEnd();

	 // sotto
	glBindTexture(GL_TEXTURE_2D, texCube[TexCubeYneg]);
	glBegin(GL_QUADS);		
		glTexCoord2f(uno, uno); glVertex3f(x,		  y,		z);
		glTexCoord2f(uno, zero); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(zero, zero); glVertex3f(x+width, y,		z+length); 
		glTexCoord2f(zero, uno); glVertex3f(x+width, y,		z);
	glEnd();

	glPopMatrix();
	glDisable (GL_TEXTURE_2D);
}



//carica il modello .ply della macchina
void loadCarMesh(){
	// Generic loading of the mesh from disk
	if(vcg::tri::io::Importer<CMesh>::Open(mesh,CAR_MODEL_PATH)!=0) {
		fprintf(stderr,"Error reading file %s\n",CAR_MODEL_PATH);
		exit(0);
	}

	// Initialize the mesh itself
	vcg::tri::UpdateBounding<CMesh>::Box(mesh);      // update bounding box
	vcg::tri::UpdateNormals<CMesh>::PerVertexPerFace(mesh); // update Normals
		
	// Initialize the wrapper
	glWrap.m = &mesh;
	glWrap.Update();

}


//init Flares
float red[3] = {1.0, 0.0, 0.0};
float green[3] = {0.0, 1.0, 0.0};
float blue[3] = {0.0, 0.0, 1.0};
float white[3] = {1,1,1};
void init_flares(){
	Fl = new Flares(10);
	//texture, distanza(piccolo=+vicino alla macchina), fattore di scala, colore, fattore di scala x il colore
	Fl->F[0].SetFlare(texFlare[TexFlare3],1.0,0.1,white,0.4);
	Fl->F[1].SetFlare(texFlare[TexFlare2],0.9,0.11,blue,0.4);
	Fl->F[2].SetFlare(texFlare[TexFlare1],0.8,0.16,green,0.4);
	Fl->F[3].SetFlare(texFlare[TexFlare3],0.7,0.08,white,0.5);
	Fl->F[4].SetFlare(texFlare[TexFlare2],0.6,0.09,red,0.3);
	Fl->F[5].SetFlare(texFlare[TexFlare2],0.5,0.08,white,0.3);
	Fl->F[6].SetFlare(texFlare[TexFlare3],0.4,0.2,blue,0.2);
	Fl->F[7].SetFlare(texFlare[TexFlare1],0.3,0.15,green,0.4);
	Fl->F[8].SetFlare(texFlare[TexFlare3],0.2,0.08,white,0.2);
	Fl->F[9].SetFlare(texFlare[TexFlare2],0.1,0.06,white,0.4);
}
void init_lights(){
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0); //sole
  glEnable(GL_LIGHT1); //fari della macchina
  glEnable(GL_LIGHT2);

	//glEnable(GL_NORMALIZE);
}
void set_lights(){
	/* setta i parametri del materiale */
	glMaterialfv(GL_FRONT,GL_EMISSION, &mat_lig[0]);
	glMaterialfv(GL_FRONT,GL_AMBIENT, &mat_lig[4]);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, &mat_lig[8]);
	glMaterialfv(GL_FRONT,GL_SPECULAR, &mat_lig[12]);
	glMaterialf(GL_FRONT,GL_SHININESS,100); 
	/* setta i parametri della luce */
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,&mat_lig[16]);
	glLightfv(GL_LIGHT0,GL_AMBIENT,  &mat_lig[20]);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,  &mat_lig[24]);
	glLightfv(GL_LIGHT0,GL_SPECULAR, &mat_lig[28]);
		
	vcg::Point4f sun=race.circuit->sun_position;
	glLightfv(GL_LIGHT0,GL_POSITION,&sun[0]);


		GLfloat light_spot[]={0.0,0.6,1.0,1.0}; //colore spotlight
		//spotlight dx
		glLightfv(GL_LIGHT1,GL_AMBIENT,  &mat_lig[20]);
		glLightfv(GL_LIGHT1,GL_DIFFUSE,  &mat_lig[24]);
		glLightfv(GL_LIGHT1,GL_SPECULAR, &mat_lig[28]);	
		glLighti(GL_LIGHT1,GL_SPOT_EXPONENT, 50);	 
		glLightf(GL_LIGHT1,GL_SPOT_CUTOFF, 30 );
		glLightf (GL_LIGHT1, GL_LINEAR_ATTENUATION,    0.02f);
		glLightf (GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.0f);
		glLightf (GL_LIGHT1, GL_CONSTANT_ATTENUATION,  0.0); 
		glLightfv(GL_LIGHT1, GL_AMBIENT, light_spot); 
		//spotlight sx
  		glLightfv(GL_LIGHT2,GL_AMBIENT,  &mat_lig[20]);
		glLightfv(GL_LIGHT2,GL_DIFFUSE,  &mat_lig[24]);
		glLightfv(GL_LIGHT2,GL_SPECULAR, &mat_lig[28]);	
		glLighti(GL_LIGHT2,GL_SPOT_EXPONENT, 50 );	 
		glLightf(GL_LIGHT2,GL_SPOT_CUTOFF, 30 );
		glLightf (GL_LIGHT2, GL_LINEAR_ATTENUATION,    0.02f);
		glLightf (GL_LIGHT2, GL_QUADRATIC_ATTENUATION, 0.0f);
		glLightf (GL_LIGHT2, GL_CONSTANT_ATTENUATION,  0.0);
		glLightfv(GL_LIGHT2, GL_AMBIENT, light_spot);

}
/** inizializzazione lo stato di opengl */
void init_opengl(){
		initTexture();
		loadCarMesh();
		init_lights();
		init_flares();
		x=0;
		y=0;
		z=0;
		DEPTH_RENDER=1;
		incr_focus_dist=0;
	glPolygonMode(GL_FRONT,GL_FILL); // rasterizza solo i contorni dei poligoni
	glEnable (GL_TEXTURE_2D);
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);

	int i=0;
	//MATERIALE
	for(i=0; i < 4; ++i) mat_lig[i]=0.0; //EMISSIVA
	for(i=4; i < 8; ++i) mat_lig[i]=0.4; //AMBIENTE
	for(i=8; i < 12; ++i) mat_lig[i]=0.6; //DIFFUSA
	for(i=12; i < 16; ++i) mat_lig[i]=0.0; //SPECULARE

	for(i=16; i < 20; ++i) mat_lig[i]=0.0; //EMISSIVA
	for(i=20; i < 24; ++i) mat_lig[i]=0.4; //AMBIENTE
	for(i=24; i < 28; ++i) mat_lig[i]=0.6; //DIFFUSA
	for(i=28; i < 32; ++i) mat_lig[i]=0.0; //SPECULARE

   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClearAccum(0.0, 0.0, 0.0, 0.0); 

}

/** callback function for viewport resize.*/
void resize_screen(int w, int h){

	width = w;
	height = h;
	glViewport(0,0,width,height);
}


/** draw reference system*/
void draw_axis(){

	glPushAttrib(0xffffffff);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1.0,0.0,0.0);
  glVertex3f(0,0,0); //   X
  glVertex3f(10,0,0); 
  
	glColor3f(0.0,1.0,0.0);
  glVertex3f(0,0,0); //   Y
  glVertex3f(0,10,0);
  
	glColor3f(0.0,0.0,1.0);
  glVertex3f(0,0,0); //   Z
  glVertex3f(0,0,10);
  glEnd();
 	glPopAttrib();
	glEnable(GL_LIGHTING);
}




void   draw_car_model(){
    glMatrixMode(GL_MODELVIEW);
	glDisable(GL_LIGHTING);

	glPushMatrix();
		float d=mesh.bbox.Diag();
		glScale(8/d);
		glRotatef(-90,1,0,0);
		glRotatef(180,0,0,1);					
		glTranslate(-glWrap.m->bbox.Center());	
		glWrap.Draw<vcg::GLW::DMSmooth,   vcg::GLW::CMPerFace ,vcg::GLW::TMNone> (); //x vedere i colori macchina
		//glWrap.Draw<vcg::GLW::DMSmooth,   vcg::GLW::CMNone,vcg::GLW::TMNone> ();
	glPopMatrix();

	//metto 2 sfere dove partono i fari
		glPushMatrix();
			glTranslatef(-0.9,0.1,-2.5);
			glutSolidSphere(0.2,5,5);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.9, 0.1, -2.5);
			glutSolidSphere(0.2,5,5);
		glPopMatrix();
		
		glEnable(GL_LIGHTING);
		//setto le posizioni dei fari, solidali con la macchina
		vcg::Point4f spotpos = vcg::Point4f ( -0.9 , 0.3 , -2.5 ,1.0);
		vcg::Point4f spotpos2 = vcg::Point4f ( 0.9, 0.3, -2.5, 1.0);
		vcg::Point4f spotdir=vcg::Point4f (0.0, 0.0, -1.0, 0.0);
		glLightfv(GL_LIGHT1,GL_POSITION,&spotpos[0]);
		glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION, &spotdir[0] );
		glLightfv(GL_LIGHT2,GL_POSITION,&spotpos2[0]);
		glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION, &spotdir[0] );
		
		//sfere x valutare la profondit di campo
		//disable_cube_map();
		//glPushMatrix();
		//	glColor3f(1.0,0.0,0.0);
		//	glTranslatef(-6.0,3.0,10.0);
		//	glutSolidSphere(1,10,10);
		//glPopMatrix();
		//glPushMatrix();
		//	glColor3f(1.0,1.0,0.0);
		//	glTranslatef(-3.0,3.0,5.0);
		//	glutSolidSphere(1,10,10);
		//glPopMatrix();
		//glPushMatrix();
		//	glColor3f(0.0,1.0,0.0);
		//	glTranslatef(0.0,3.0,0.0);
		//	glutSolidSphere(1,10,10);
		//glPopMatrix();
		//glPushMatrix();
		//	glColor3f(0.0,1.0,1.0);
		//	glTranslatef(3.0,3.0,-5.0);
		//	glutSolidSphere(1,10,10);
		//glPopMatrix();
		//glPushMatrix();
		//	glColor3f(0.0,0.0,1.0);
		//	glTranslatef(6.0,3.0,-10.0);
		//	glutSolidSphere(1,10,10);
		//glPopMatrix();

}


GLuint compileCarDL(){
	GLuint carDL=glGenLists(1);
	glNewList(carDL,GL_COMPILE);
	draw_car_model();
	glEndList();
	return carDL;
}
void gl_print(float x, float y, const char *str) {
  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
}


void applyRoadTexture(vcg::Point3f *p00, vcg::Point3f *p10, vcg::Point3f *p01, vcg::Point3f *p11 ){
	glTexCoord2f(0.0, 0.0); glVertex3f(p00->X(), p00->Y()+0.1, p00->Z()+10);//basso dx
	glTexCoord2f(1.0, 0.0); glVertex3f(p10->X(), p10->Y()+0.1, p10->Z()+10);//basso sx
	glTexCoord2f(0.0, 1.0); glVertex3f(p01->X(), p01->Y()+0.1, p01->Z()+10);//alto dx
	glTexCoord2f(1.0, 1.0); glVertex3f(p11->X(), p11->Y()+0.1, p11->Z()+10);//alto sx
}

GLuint compileRoadDL(){
	GLuint roadDL = glGenLists(1);
	glNewList (roadDL,GL_COMPILE);
	Street::TilesIterator ti;
	int i;
	glColor3f(0.5,0.5,0.5); //colore x la strada
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glBindTexture( GL_TEXTURE_2D, texName[TexRoad]);
	glBegin(GL_QUAD_STRIP);
	glNormal3f(0,1,0);
	//glCullFace(GL_FRONT);
	vcg::Point3f first0,first1;
	for(i=0,ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end();++ti,++i){
		vcg::Point3f p0,p1;
		p0 = (*ti)[0].P(); //p0 sono i vertici dx del circuito
		p1 = (*ti)[1].P(); //p1 sono i vertici sx del circuito
		if(i==0){first0=p0;first1=p1;}

		glVertex3f(p0[0],p0[1]+0.1,p0[2]+10);
		glVertex3f(p1[0],p1[1]+0.1,p1[2]+10);

		vcg::Point3f p01,p11;
		if(ti!=race.circuit->street.path.end()-1){
			++ti;
			p01= (*ti)[0].P();//dx sopra
			p11= (*ti)[1].P();//sx sopra
			applyRoadTexture(&p0,&p1,&p01,&p11);
			--ti;
		}
		else{
			applyRoadTexture(&p0,&p1,&first0,&first1);
		}
	}

	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);

	glEndList();
	return roadDL;
}

//buildings
void draw_buildings(){
	glEnable(GL_NORMALIZE);
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	Circuit::BuildingsIterator bi; //iteratore sui palazzi

	float height=0.0; //altezza palazzo
	int cont=0;
	for(bi=race.circuit->buildings.begin();bi!=race.circuit->buildings.end();++bi){
		//alterno le 3 texture x i palazzi
		if(cont%3==0)
			glBindTexture( GL_TEXTURE_2D, texName[TexBuilding1]);
		else
			if(cont%3==1)
					glBindTexture(GL_TEXTURE_2D, texName[TexBuilding3]);
				else
					glBindTexture( GL_TEXTURE_2D, texName[TexBuilding2]);
		height=(*bi).height;
		std::vector<Vertex>::iterator vi; //iteratore sui vertici di ciascun palazzo
		for(vi=(*bi).footprint.begin(); vi!=(*bi).footprint.end()-1;++vi){
			vcg::Point3f p00 = (*vi).P();//x0
			++vi;
			vcg::Point3f p01 = (*vi).P();//y0
			--vi;
			vcg::Point3f ver = vcg::Point3f(0,height,0);
			vcg::Point3f hor = vcg::Point3f(p01[0]-p00[0],p01[1]+height-p00[1],p01[2]-p00[2]);
			vcg::Point3f normal = ver^hor;
			normal=normal.Normalize();
			glNormal3f(normal[0],normal[1],normal[2]);
			
			glBegin(GL_QUADS);
				glTexCoord2f(0.0,0.0);glVertex3f(p00[0],p00[1],p00[2]);
				glTexCoord2f(25.0,0.0);glVertex3f(p01[0],p01[1],p01[2]);
				glTexCoord2f(25.0,2.0);glVertex3f(p01[0],p01[1]+height,p01[2]);
				glTexCoord2f(0.0,2.0);glVertex3f(p00[0],p00[1]+height,p00[2]);				
			glEnd();
		}

		glBegin(GL_QUADS);
			vi=(*bi).footprint.end()-1;
			vcg::Point3f p00=(*vi).P();
			glTexCoord2f(0.0,0.0);glVertex3f(p00[0],p00[1],p00[2]);
			vi=(*bi).footprint.begin();
			vcg::Point3f p01=(*vi).P();
			glTexCoord2f(25.0,0.0);glVertex3f(p01[0],p01[1],p01[2]);
			glTexCoord2f(25.0,2.0);glVertex3f(p01[0],p01[1]+height,p01[2]);
			glTexCoord2f(0.0,2.0);glVertex3f(p00[0],p00[1]+height,p00[2]);
		glEnd();
		cont++;
	}

	glDisable(GL_TEXTURE_2D);
	}



//billboarding
void billboard(float *cam, float *worldPos) {
	
	//il quad guarda verzo +z
	Point3f lookAt = vcg::Point3f(0,0,1);

	//vettore dalla camera all'albero, proiettato su XZ
	Point3f objToCamProj = vcg::Point3f(cam[0]-worldPos[0],0,cam[2]-worldPos[2]);
	objToCamProj=objToCamProj.Normalize();

	//vettore dalla camera all'albero
	Point3f objToCam = vcg::Point3f(cam[0]-worldPos[0],cam[1]-worldPos[1],cam[2]-worldPos[2]);
	objToCam=objToCam.Normalize();

	//prodotto vettoriale tra objToCamProj e lookAt --> mi d l'orientazione dell'angolo (pos o neg)
	// se l'angolo  positivo upAux  un vettore che punta nella direzione positiva delle y
	// altrimenti punter in basso e la rotazione sar invertita
	Point3f upAux = lookAt^objToCamProj;

	// calcolo il prodotto scalare tra i 2 vettori == cos(angolo)
	float coseno=lookAt*objToCamProj;
	
	// rotazione 
	// stability reasons
	// if the lookAt and v vectors are too close together then |aux| could
	// be bigger than 1 due to lack of precision
	if ((coseno < 0.99990) && (coseno > -0.9999))
		glRotatef(acos(coseno)*180/3.14,upAux[0], upAux[1], upAux[2]);	

    //INCLINAZIONE
	// prodotto scalare x trovare il coseno dell'angolo tra objToCamProj e objToCam
	// mi serve x stabilire la rotazione rispetto all'asse X --> inclinazione
	coseno = objToCamProj*objToCam;

	// rotazione attorno all'asse x per migliorare la vista dall'alto
	// The test is done to prevent instability when objToCam and objToCamProj have a very small
	// angle between them
	if ((coseno < 0.99990) && (coseno> -0.9999))
		if (objToCam[1] < 0)
			glRotatef(acos(coseno)*180/3.14,1,0,0);	
		else
			glRotatef(acos(coseno)*180/3.14,-1,0,0);	

}




/** draw trees (billboard)**/
void draw_billboard_trees(){
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glEnable(GL_BLEND);
	// il colore di ci che disegno viene pesato per alpha, il colore gi presente nel buffere per (1-alpha) 
	// la somma di questi 2 colori viene scritta nel framebuffer.
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_ALPHA_TEST);
	//disegno il frammento solo se il suo valore alpha  > 0
	glAlphaFunc(GL_GREATER, 0);
	glBindTexture(GL_TEXTURE_2D, texName[TexTree]);

	Circuit::TreesIterator ti;
	vcg::Point3f distance;
	for(ti=race.circuit->trees.begin();ti!=race.circuit->trees.end();++ti){
		distance = pov - ti->pos;
		if(distance.Norm()<700){ //lo disegno solo se sta prima del far-plane
		glPushMatrix();
			vcg::Point3f pos=(*ti).pos;
			float height = (*ti).height;
			float cam[3]={pov[0]+x,pov[1]+y,pov[2]+x};
			float worldPos[3]={pos[0],pos[1],pos[2]};
			glTranslatef(worldPos[0],worldPos[1],worldPos[2]);

			billboard(cam,worldPos);

			glBegin(GL_QUADS);
				glTexCoord2f(0,0);glVertex3f(-(height), 0.0f, 0.0f);
				glTexCoord2f(1,0);glVertex3f((height), 0.0f, 0.0f);
				glTexCoord2f(1,1);glVertex3f((height), height*2,  0.0f);
				glTexCoord2f(0,1);glVertex3f(-(height), height*2,  0.0f);
			glEnd();

		glPopMatrix();
		}
	}
	glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);
	glDisable(GL_TEXTURE_2D);
	
}



/* accFrustum()
 * The first 6 arguments are identical to the glFrustum() call.
 *  
 * pixdx and pixdy are anti-alias jitter in pixels. 
 * Set both equal to 0.0 for no anti-alias jitter.
 * eyedx and eyedy are depth-of field jitter in pixels. 
 * Set both equal to 0.0 for no depth of field effects.
 *
 * focus is distance from eye to plane in focus. 
 * focus must be greater than, but not equal to 0.0.
 *
 * Note that accFrustum() calls glTranslatef().  You will 
 * probably want to insure that your ModelView matrix has been 
 * initialized to identity before calling accFrustum().
 */

void accFrustum(GLdouble left, GLdouble right, GLdouble bottom, 
   GLdouble top, GLdouble near, GLdouble far, GLdouble pixdx, 
   GLdouble pixdy, GLdouble eyedx, GLdouble eyedy, GLdouble focus)
{
   GLdouble xwsize, ywsize; 
   GLdouble dx, dy;
   GLint viewport[4];

   glGetIntegerv (GL_VIEWPORT, viewport);
	
   xwsize = right - left;
   ywsize = top - bottom;
	
   dx = -(pixdx*xwsize/(GLdouble) viewport[2]+ eyedx*near/focus);
   dy = -(pixdy*ywsize/(GLdouble) viewport[3]+ eyedy*near/focus);
	
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum (left + dx, right + dx, bottom + dy, top + dy, near, far);
 
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef (-eyedx, -eyedy, 0.0);

}

/*  accPerspective()
 * 
 *  The first 4 arguments are identical to the gluPerspective() call.
 *  pixdx and pixdy are anti-alias jitter in pixels. 
 *  Set both equal to 0.0 for no anti-alias jitter.
 *  eyedx and eyedy are depth-of field jitter in pixels. 
 *  Set both equal to 0.0 for no depth of field effects.
 *
 *  focus is distance from eye to plane in focus. 
 *  focus must be greater than, but not equal to 0.0.
 *
 *  Note that accPerspective() calls accFrustum().
 */

void accPerspective(GLdouble fovy, GLdouble aspect, 
   GLdouble near, GLdouble far, GLdouble pixdx, GLdouble pixdy, 
   GLdouble eyedx, GLdouble eyedy, GLdouble focus)
{
   GLdouble fov2,left,right,bottom,top;

   fov2 = ((fovy*M_PI) / 180.0) / 2.0;

   top = near / (cos(fov2) / sin(fov2));
   bottom = -top;

   right = top * aspect;
   left = -right;

   accFrustum (left, right, bottom, top, near, far,
               pixdx, pixdy, eyedx, eyedy, focus);
}
void draw_everything(){

	glDisable(GL_LIGHTING);
	//disabilito la scrittura nel depth buffer prima di disegnare lo skybox
	glDepthMask(false);
	glPushMatrix();
		draw_skybox(pov.X(),pov.Y(),pov.Z(),4,2,4);
	glPopMatrix();
	glDepthMask(true);
	glEnable(GL_LIGHTING);

	//disegno il prato su tutto il bounding box del circuito
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glColor3f(0.6,0.6,0.6);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glBindTexture(GL_TEXTURE_2D, texName[TexGrass]);
	glBegin(GL_QUADS);	
		glTexCoord2f(0.0,0.0);
		glVertex3f(race.circuit->bbox.min[0],race.circuit->bbox.min[1],race.circuit->bbox.min[2]);
		glTexCoord2f(3000.0,0.0);
		glVertex3f(race.circuit->bbox.max[0],race.circuit->bbox.min[1],race.circuit->bbox.min[2]);
		glTexCoord2f(0.0,3000.0);
		glVertex3f(race.circuit->bbox.max[0],race.circuit->bbox.min[1],race.circuit->bbox.max[2]);
		glTexCoord2f(3000.0,3000.0);
		glVertex3f(race.circuit->bbox.min[0],race.circuit->bbox.min[1],race.circuit->bbox.max[2]);
	glEnd();
	glDisable(GL_TEXTURE_2D);

	glPushMatrix();
		glScalef(race.circuit->bbox.max[0]-race.circuit->bbox.min[0],race.circuit->bbox.max[1]-race.circuit->bbox.min[1],race.circuit->bbox.max[2]-race.circuit->bbox.min[2]);
		draw_axis();
	glPopMatrix();

	// draw circuit	
	glCallList(roadDL);

	//draw bulidings
	draw_buildings();

	//draw trees
	draw_billboard_trees();

	// draw players
	Race::PlayersIterator pi;
	int st = clock();
	for(pi = cli.players.begin(); pi != cli.players.end(); ++pi)
		{
			glPushMatrix();
				//glMateriali(GL_FRONT,GL_SHINESS,50)
				/* apply transformation coded in the frame */
				Frame f = ((*pi).car.frame);	

				//calcolo l'inversa (trasposta) del frame della macchina
				float inv[4][4];
				unsigned int i, j;
				for (i=0; i<4; i++)
					for (j=0; j<4; j++)
						inv[i][j] = f.ElementAt(j,i);

				glMultMatrixf(&f[0][0]); 
				glPushMatrix();
					glScalef(0.5,0.5,0.5);
					glTranslatef(0.0,2.0,0.0);

				   /*GL_TEXTURE_BIT	   Enable bits for the four texture coordinates
				   Border color	for each texture image,
				   Minification	function and Magnification function for each texture image
				   Texture coordinates and wrap	mode for each texture image
				   Color and mode for each texture environment*/

					//X DYNAMIC CM.
					glPushAttrib(GL_TEXTURE_BIT); //salvo le impostazioni x le texture

					//ablito il cube mapping solo sulla macchina
					enable_cube_map();

					//X DYNAMIC CM.
					glPushMatrix();					
					glMatrixMode(GL_TEXTURE);					
					glPushMatrix();
					//prima di applicare il cubemap, moltiplico la matrice delle texture x l'inversa del frame
					glMultMatrixf(&inv[0][0]);  
					glMatrixMode(GL_MODELVIEW);

					//glutSolidTeapot(3);
					glCallList(carDL);
					
					//X DYNAMIC CM.
					glMatrixMode(GL_TEXTURE);
					glPopMatrix();//ripristino la matrice texture
					glMatrixMode(GL_MODELVIEW);
					glPopMatrix(); 
					glPopAttrib();
					glFlush();
					
					//disabilito il cube mapping
					disable_cube_map();
				glPopMatrix();
				glDisable(GL_LIGHTING);
				glColor3f(1.0,0.0,0.0);
				gl_print(0.0,2.0,(*pi).ip_address.c_str()); // write the IP address of the client
				glEnable(GL_LIGHTING);
			glPopMatrix();
	 }

	//calcolo parametri x abilitare il lens-flare
	vcg::Point3f from(pov.X(),pov.Y(),pov.Z());
	vcg::Point3f at(position.X(),position.Y(),position.Z());
	vcg::Point3f pos(race.circuit->sun_position[0],race.circuit->sun_position[1],race.circuit->sun_position[2]);
	Fl->DoFlares(from,at,pos,2);

	glLoadIdentity();	
}

/** render the scene */
void draw_scene(){

	static int cn = 0;
	static int part = 0;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	set_lights();

	SDL_Delay(40);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* here take the bounding box of the whole circuit */
	vcg::Point3f bmin = race.circuit->bbox.min;
	vcg::Point3f bmax = race.circuit->bbox.max;
	float sidex,sidez;
	sidex = bmax[0]-bmin[0];
	sidez = bmax[2]-bmin[2];

	/* updte the state of the players as received by the server */
	cli.UpdatePlayers();

	//glOrtho(-sidex*0.5,sidex*0.5,-sidez*0.5,sidez*0.5,0.0,400);
	Race::PlayersIterator fpi;
	vcg::Point3f delta=vcg::Point3f(0,0,0);;
	Frame myf;
	if(cli.players.size()>0){
		fpi = cli.players.begin();
		myf=(*fpi).car.frame; //frame della macchina, ricevuto dal server
		position=myf.GetRow3(3); //la mia posizione (quarta riga frame)
		//di quanto mi sposto dietro e sopra la macchina (x calcolare il pov)
		delta=myf.GetRow3(2)*10+myf.GetRow3(1)*3.5;
	}

	//gluPerspective(75,width/(float)height,0.1,700);

	pov=position + delta; //pov x la lookAt
	pov=pov+myf.GetRow3(0)*x+myf.GetRow3(1)*y+myf.GetRow3(2)*z;

	//draw_everything();

   GLint viewport[4];
   glGetIntegerv (GL_VIEWPORT, viewport);

   cout<<"position "<<position.X()<<" "<<position.Y()<<" "<<position.Z()<<endl;
   
   //ripulitura accumulation buffer
   glClear(GL_ACCUM_BUFFER_BIT);

	//calcolo il frame di vista
	Point3f z_f = (pov-position).Normalize();
	Point3f x_f = (z_f^vcg::Point3f(0,1,0)).Normalize();
	Point3f y_f = (x_f^z_f).Normalize();
	//cout<<"view-frame"<<endl;
	//cout<<x_f[0]<<" "<<x_f[1]<<" "<<x_f[2]<<endl;
	//cout<<y_f[0]<<" "<<y_f[1]<<" "<<y_f[2]<<endl;
	//cout<<x_f[0]<<" "<<x_f[1]<<" "<<x_f[2]<<endl;

   //distanza del piano a fuoco: pov <--> car
   original_dist = sqrt (pow(position.X()-pov.X(),2)+pow(position.Y()-pov.Y(),2)+pow(position.Z()-pov.Z(),2));
   dist = original_dist + incr_focus_dist;
   //cout<<dist<<endl;
   for (int i = 0; i < DEPTH_RENDER ; i++) {
	  //cout<<"RENDERING "<<i<<endl;
	  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	  glMatrixMode(GL_PROJECTION);
	  glLoadIdentity();

	  accPerspective (75.0, 
         (GLdouble) viewport[2]/(GLdouble) viewport[3], 
         0.1, 700.0, 0.0, 0.0,
		 0.25*j4[i].x, 0.25*j4[i].y , dist);

	  pov=pov+x_f*(0.25*j4[i].x)+y_f*(0.25*j4[i].y);
	  
	  glMatrixMode(GL_MODELVIEW);
	  gluLookAt(pov.X(),pov.Y(),pov.Z(),position.X(),position.Y(),position.Z(),0,1,0);
	  
	  draw_everything();

	  //sommo il rendering nell'accumulation buffer
      glAccum (GL_ACCUM, 1.0/(float)DEPTH_RENDER);
   }
 
   glAccum (GL_RETURN, 1.0);
   glFlush();

	SDL_GL_SwapBuffers(); //scambio buffer 

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	/* print the commands */
	gl_print(-1,0.9,"accel: w");
	gl_print(-1,0.8,"retro:  s");
	gl_print(-1,0.7,"left:  a");
	gl_print(-1,0.6,"right: d");
	gl_print(-1,0.5,"brake: x");
	gl_print(-1,0.4,"reset: p");
	gl_print(-1,0.3,"DOF on/off: F1");
	gl_print(-1,0.2,"DOF-focus: c,v,b");
	gl_print(-1,0.1,"Cam: (u,i)(j,k)(n,m)-r");
	glEnable(GL_TEXTURE_2D);

	SDL_GL_SwapBuffers();
	cn++;
	if(clock()-part >1000){	
		printf("VIEW fps %d\n",cn);
		part = clock();
		cn=0;
	}
}


/// callback function che gestisce l'evento "mouse mosso"
void mouse_move(SDL_Event e){
}
///
void keydown(SDL_Event e){
	switch(e.key.keysym.sym){
		case SDLK_w: cli.Command(COMMAND_INCREASE_SPEED);break;
		case SDLK_s: cli.Command(COMMAND_DECREASE_SPEED);break;
		case SDLK_a: cli.Command(COMMAND_TURN_LEFT);break;
		case SDLK_d: cli.Command(COMMAND_TURN_RIGHT);break;
		case SDLK_x: cli.Command(COMMAND_BRAKE);break;
		case SDLK_p: cli.Command(COMMAND_RESTART);break;

		case SDLK_u: x++; break; 
		case SDLK_i: x--; break;
		case SDLK_j: y++; break;
		case SDLK_k: if(pov.Y()>1)y--; break;
		case SDLK_n: z++; break;
		case SDLK_m: z--; break;
		case SDLK_r: x=0;y=0;z=0;break; //ripristino pov iniziale

		case SDLK_F1: DEPTH_RENDER = (DEPTH_RENDER == 4 ? 1 : 4); break;

		case SDLK_c: if(DEPTH_RENDER>1)incr_focus_dist+=0.5;break;
		case SDLK_v: if(DEPTH_RENDER>1 && dist>original_dist-4)incr_focus_dist-=0.5;break;
		case SDLK_b: incr_focus_dist=0;break;

	}
}

int launch_viewer(){
  // inizializzazione di SDL  
  SDL_Init( SDL_INIT_VIDEO ); 

  //attivazione accumulation buffer
  SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE,16);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE,16);

  SDL_SetVideoMode(650,650,0, SDL_OPENGL);

  // inizializzamo lo stato di opengl
  init_opengl();

  //inizializzazione cube mapping (load textures)
  init_cube_map();

  //compilo le display-list
  carDL=compileCarDL();
  roadDL=compileRoadDL();

  // settiamo OpenGL alle dimensioni 650x650
  resize_screen(650,650);
  
  static int last_rendering = 0;
  int done=0;
  while (!done) { // loop gestione eventi
    
    SDL_Event e;
 
		// guarda se c' un evento. Che ci sia o no continua
    if(SDL_PollEvent(&e))
			// processo l'evento
			switch (e.type) {
				case SDL_KEYDOWN: 
					  keydown(e);
					break;
				case SDL_VIDEORESIZE:  
					// evento: a dim della finestra e' cambiata
					resize_screen(e.resize.w,e.resize.h);
					break;
	        
				case SDL_QUIT:  
					// evento: la applicazione deve uscire
					done=1;   
					break;
				}

		/* temporizzazione del rendering: rendere il refresh machine independent*/
			draw_scene();	

  }
	return 0;
}

