/****************************************************************************
*                                                                           *
* Copyright(C) 2007                                                         *
* Pierluigi Failla                                                          *
*																			*
* http://www.zeta.xm.com													*
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

struct Point3D{
	public: 
		float x;
		float y;
		float z;
	
	public:
		Point3D(void); //builder
		Point3D(float _x, float _y, float _z); //parametrized builder
};

class Particle{
	public:
		float Life; //particle life
		float Fade; //fade speed
		Point3D Speed; //particle speed x,y,z
		Point3D Angle; //particle angle x,y,z
		Point3D Position; //particle position
		Point3D Direction; //particle direction

	public:
		Particle(void); //builder
};

class Particles{
	public:
		int MAX_PARTICLES; //max num of particles
		Particle *SysPa; //vector of particle
		float SpreadFactor;
		bool Distrib3D; //quad distribution 
		
	private:	
		GLuint Tex;
		float SlowDownFactor;

	public:
		Particles(void); //builder
		Particles(int TotParticles); //builder with max number of particles
		bool SetTexture(GLuint TextureNumber); //make a blurry texture for the particle system
		void UseThisTexture(GLuint TextureNumber); //use another texture
		void InitSystem(void); //initialize system
		void DrawSystem(void); //draw
		void DirectionRule(int ParticleNumber);
		void SpeedRule(int ParticleNumber);
		void AngleRule(int ParticleNumber);
		void PositionRule(int ParticleNumber);
		void CheckSpread_SQRT(int ParticleNumber);
		void CheckSpread_ABS(int ParticleNumber);
};