#include "myfunctions.h"

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <SDL.h>
#include <GL/glew.h>
#include <math.h>
#include <time.h>
#include <vcg/space/point3.h>
#include <vector>
#include <GL/glut.h>
#include <GL/glu.h>

#define circuito_tex "../../client3/textures/circuito.bmp"
#define circuito_bind 1
#define erba_tex "../../client3/textures/erba1a.bmp"
#define erba_bind 2
#define palazzo_tex "../../client3/textures/palazzo3a.bmp"
#define palazzo_bind 3

using namespace vcg;
using namespace std;

void gl_print(float x, float y, const char *str) {
  glDisable(GL_TEXTURE_2D);
  glRasterPos2f(x, y);
  int len = (int)strlen(str);
  for(int i = 0; i < len; i++) 
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, str[i]);
}

//DISEGNA CIRCUITO
void draw_circuito(){
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glBindTexture( GL_TEXTURE_2D, circuito_bind );

	Street::TilesIterator ti;
	//int i;
	vcg::Point3f p0,p1;
	vcg::Point3f first0,first1;
	
	glBegin(GL_QUAD_STRIP);
	// with following code we loose the last quad
	for(ti = race.circuit->street.path.begin(); ti != race.circuit->street.path.end()-1;++ti){
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(0.0,0.0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(1.0,0.0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		++ti;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(0.0,1.0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(1.0,1.0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		--ti;
	}
	glEnd();
	//street closure the last quad
	glCullFace(GL_BACK);
	glBegin(GL_QUADS);
		ti = race.circuit->street.path.end();
		--ti;
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(1,0);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(0,0);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
		ti = race.circuit->street.path.begin();
		p0 = (*ti)[0].P(); p1 = (*ti)[1].P();
		glTexCoord2d(1,1);
		glVertex3f(p0[0],p0[1]+0.01,p0[2]);
		glTexCoord2d(0,1);
		glVertex3f(p1[0],p1[1]+0.01,p1[2]);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}
// APPLICA LA TEXTURE DEL CIRCUITO
void applyRoadTexture(vcg::Point3f *p00, vcg::Point3f *p10, vcg::Point3f *p01, vcg::Point3f *p11 ){
	glTexCoord2f(0.0, 0.0); glVertex3f(p00->X(), p00->Y()+0.1, p00->Z()+10);//basso dx
	glTexCoord2f(1.0, 0.0); glVertex3f(p10->X(), p10->Y()+0.1, p10->Z()+10);//basso sx
	glTexCoord2f(0.0, 1.0); glVertex3f(p01->X(), p01->Y()+0.1, p01->Z()+10);//alto dx
	glTexCoord2f(1.0, 1.0); glVertex3f(p11->X(), p11->Y()+0.1, p11->Z()+10);//alto sx
}
// DISEGNA ERBA
void draw_erba(){
	glEnable(GL_TEXTURE_2D);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glBindTexture( GL_TEXTURE_2D, erba_bind );	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0,0.0);
		glVertex3f(race.circuit->bbox.min[0],race.circuit->bbox.min[1],race.circuit->bbox.min[2]);
		glTexCoord2f(3000.0,0.0);
		glVertex3f(race.circuit->bbox.max[0],race.circuit->bbox.min[1],race.circuit->bbox.min[2]);
		glTexCoord2f(0.0,3000.0);
		glVertex3f(race.circuit->bbox.max[0],race.circuit->bbox.min[1],race.circuit->bbox.max[2]);
		glTexCoord2f(3000.0,3000.0);
		glVertex3f(race.circuit->bbox.min[0],race.circuit->bbox.min[1],race.circuit->bbox.max[2]);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

// DISEGNA SKYBOX
void draw_skybox(float x, float y, float z, float width, float height, float length){
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDepthMask(GL_FALSE);
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
	
	// NUMERI SU CUI VENGONO ASSOCIATE LE TEXTURE
	int faceBind[6] = {4,5,6,7,8,9};

	 // CENTRO LO SKYBOX SULLE POSIZIONI X,Y,Z PASSATE COME PARAMETRI
	x = x - width  / 2;
	y = y - height / 2;
	z = z - length / 2;

	glPushMatrix();

	// DAVANTI
	glBindTexture(GL_TEXTURE_2D, faceBind[0]);
	glBegin(GL_QUADS);	
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x,		  y+height, z+length);
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x+width, y,		z+length);
	glEnd();

	// DIETRO
	glBindTexture(GL_TEXTURE_2D, faceBind[1]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x+width, y,		z);
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x+width, y+height, z); 
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x,		  y+height,	z);
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x,		  y,		z);
	glEnd();

	// SINISTRA
	glBindTexture(GL_TEXTURE_2D, faceBind[4]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x,		  y+height,	z);	
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x,		  y+height,	z+length); 
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x,		  y,		z);		
	glEnd();

	// DESTRA
	glBindTexture(GL_TEXTURE_2D, faceBind[5]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x+width, y,		z);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x+width, y,		z+length);
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x+width, y+height,	z+length); 
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x+width, y+height,	z);
	glEnd();

	// SOPRA
	glBindTexture(GL_TEXTURE_2D, faceBind[2]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x+width, y+height, z);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x+width, y+height, z+length); 
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x,		  y+height,	z+length);
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x,		  y+height,	z);
	glEnd();

	// SOTTO
	glBindTexture(GL_TEXTURE_2D, faceBind[3]);
	glBegin(GL_QUADS);		
		glTexCoord2f(0.999f, 0.999f); glVertex3f(x,		  y,		z);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(x,		  y,		z+length);
		glTexCoord2f(0.001f, 0.001f); glVertex3f(x+width, y,		z+length); 
		glTexCoord2f(0.999f, 0.001f); glVertex3f(x+width, y,		z);
	glEnd();

	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}


float dist2D(float x, float y){
	return (float)sqrt(x*x+y*y);
}
// DISEGNA PALZZI
void draw_palazzi(){
	glEnable(GL_CULL_FACE); 
	glCullFace(GL_BACK); 
	glEnable (GL_TEXTURE_2D); 
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST); 
	glBindTexture( GL_TEXTURE_2D, palazzo_bind); 

	Circuit::BuildingsIterator bi; 
	float h = 0.0, x1 = 0.0, y1 = 0.0, x2 = 0.0, y2 = 0.0, dist = 0.0;
	for(bi = race.circuit->buildings.begin(); bi != race.circuit->buildings.end(); ++bi){
		h = (*bi).height;
		for(int i=0; i < (*bi).footprint.size()-1;++i){
			glBegin(GL_QUADS); 
				glTexCoord2d(0,0);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
				x1 = (float)(*bi).footprint[i].P()[0];
				y1 = (float)(*bi).footprint[i].P()[2];
				++i;
				x2 = (float)(*bi).footprint[i].P()[0];
				y2 = (float)(*bi).footprint[i].P()[2];
				dist = dist2D(x2-x1,y2-y1);
				dist = dist/10;
				glTexCoord2d(dist,0);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
				glTexCoord2d(dist,1);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
				--i;
				glTexCoord2d(0,1);
				glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
			glEnd();
		}
		glCullFace(GL_FRONT);
		int i = (*bi).footprint.size()-1;
		glBegin(GL_QUADS); //The last face of the buildings!!!
			glTexCoord2d(0,0);
			glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1],(*bi).footprint[i].P()[2]);
			x1 = (float)(*bi).footprint[i].P()[0];
			y1 = (float)(*bi).footprint[i].P()[2];
			x2 = (float)(*bi).footprint[0].P()[0];
			y2 = (float)(*bi).footprint[0].P()[2];
			dist = dist2D(x2-x1,y2-y1);
			dist = dist/10;
			glTexCoord2d(dist,0);
			glVertex3f((*bi).footprint[0].P()[0],(*bi).footprint[0].P()[1],(*bi).footprint[0].P()[2]);
			glTexCoord2d(dist,1);
			glVertex3f((*bi).footprint[0].P()[0],(*bi).footprint[0].P()[1] + h,(*bi).footprint[0].P()[2]);
			glTexCoord2d(0,1);
			glVertex3f((*bi).footprint[i].P()[0],(*bi).footprint[i].P()[1] + h,(*bi).footprint[i].P()[2]);
		glEnd();
	}

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
}
// CARICAMENTO TEXTURE DA FILE BMP
GLuint LoadTextureBMP( const char * filename, GLuint texture, int wrap, int alpha ){
	SDL_Surface *surface;
	if ((surface = SDL_LoadBMP(filename))) {
		 glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		// ALLOCA LA TEXTURE
		glGenTextures( 1, &texture );
		// BINDA LA TEXTURE
		glBindTexture( GL_TEXTURE_2D, texture );
		// MINIFICATION
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST );
		// MAGNIFICATION
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		// GL_REPEAT => RIPETIZIONE DELLA TEXTURE
		// GL_CLAMP  => ADEGUA LA TEXTURE ALLA SUPERFCIE SU CUI LA DEVE ATTACCARE
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap ? GL_REPEAT : GL_CLAMP );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap ? GL_REPEAT : GL_CLAMP );
		// CARICA LA TEXTURE
		if(alpha==GL_FALSE)
			// TEXTURE RGB SENZA CANALE ALPHA
			gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB, surface->w, surface->h, GL_BGR_EXT, GL_UNSIGNED_BYTE, surface->pixels );
		else
			// TEXTURE RGBA CON CANALE ALPHA
			gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, surface->w, surface->h, GL_BGRA_EXT, GL_UNSIGNED_BYTE, surface->pixels );
	} 
	else{
		printf("SDL could not load image.bmp: %s\n", SDL_GetError());
		SDL_Quit();
		return -1;
	} 
	if ( surface ){ 
		SDL_FreeSurface( surface );
	}
    return texture;
}

// BILLBOARDING
void billboard(float *cam, float *treePos) {
	
	// IL QUAD GUARDA VERSO +Z (VERSO DI ME)
	Point3f lookAt = vcg::Point3f(0,0,1);

	// PROIEZIONE SU XZ DEL VETTORE CAMERA-ALBERO
	Point3f objToCamProj = vcg::Point3f(cam[0]-treePos[0], 0, cam[2]-treePos[2]);
	objToCamProj = objToCamProj.Normalize();

	//VETTORE CAMERA-ALBERO
	Point3f objToCam = vcg::Point3f(cam[0]-treePos[0], cam[1]-treePos[1], cam[2]-treePos[2]);
	objToCam = objToCam.Normalize();

	// PRODOTTO VETTORIALE TRA objToCamProj E lookAt => MI DICE SE L'ANGOLO DI ROTAZIONE E' POSITIVO O NEGATIVO
	// ANGOLO POSITIVO => upAux PUNTA NELLA DIREZIONE POSITIVA DELLE Y
	// ALTRIMENTI PUNTER IN BASSO E LA ROTAZIONE SAR INVERTITA
	//Point3f upAux = lookAt^objToCamProj;
	Point3f upAux = Point3f(0,1,0);

	// CALCOLO IL PRODOTTO SCALARE TRA I 2 VETTORI => cos(angolo)
	float angleCosine = lookAt*objToCamProj;
	
	// ROTAZIONE 
	// if the lookAt and v vectors are too close together then |aux| could
	// be bigger than 1 due to lack of precision
	if ((angleCosine < 0.99990) && (angleCosine > -0.9999))
		glRotatef(acos(angleCosine)*180/3.14, upAux[0], upAux[1], upAux[2]);	

	// COSENO DELL'ANGOLO TRA objToCamProj E objToCam
	// ROTAZIONE RISPETTO ALL'ASSE X (VISTA DALL'ALTO)
	angleCosine = objToCamProj*objToCam;

	// ROTAZIONE ATTORNO ALL'ASSE X PER MIGLIORARE LA VISTA DALL'ALTO
	//if ((angleCosine < 0.99990) && (angleCosine> -0.9999))
	//	if (objToCam[1] < 0)
	//		glRotatef(acos(angleCosine)*180/3.14,1,0,0);	
	//	else
	//		glRotatef(acos(angleCosine)*180/3.14,-1,0,0);	

}