/****************************************************************************
* NVMC                                                                      *
* Visual and Computer Graphics Library                                      *
*                                                                           *
* Copyright(C) 2007                                                         *
* ISTI - Italian National Research Council                                  *
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: process_message.cpp,v $
Revision 1.1  2007/04/09 23:08:42  ganovelli
added standalone mode


****************************************************************************/

#include <process_message.h>
#include <messages.h>
#include <packaging.h>
#include "race_simulation.h"

void GetNewFrame(Frame & f){f.SetIdentity(); *(vcg::Point3f*)&f[3][0] = race.circuit->street.entry_points[0];}
int ProcessMessageServer(pt::string msg,pt::string host){
	/*first draft: the messages are only commands. The subscribtion to the game
		is automatic at the first command */
	Race::PlayersIterator pi;
	int st=clock();
	race.mutex_on_players.enter();
	pi =race.FindPlayer(std::string(host));
	if(pi == race.players.end())
		{//its a new player
 			pi = race.AddPlayer();
			(*pi).ip_address = host;
			//position the car 
			GetNewFrame((*pi).car.frame);	
			(*pi).car.vel = vcg::Point4f(0.0,0.0,0.0,0.0);
			(*pi).car.acc = vcg::Point4f(0.0,0.0,0.0,0.0);
			(*pi).car.steering_angle = 0.0;
			(*pi).car.back_wheel_pos = 1.0;
			(*pi).car.front_wheel_pos = -1.0;
			(*pi).car.max_speed = 100; // 360 km/h
			(*pi).state = Player::PLAYING;
		}
	race.mutex_on_players.leave();

	/* now queue the action */
	mutex_on_actionsToDo.enter();
	actionsToDo.push_back(Action((*pi).ip_address,UnPack(msg)));
	mutex_on_actionsToDo.leave();
	//printf("SERVER: received %s!\n",msg);
	printf("message processed in %d\n",clock()-st);	


return 0;
}

/** process the message: first draft, the message is:
gTime, list of (ip addresses,Frame) */
int ProcessMessagePlayer(pt::string msg,pt::string host){

	/* we dont't care about the host, in this version it can only be server */
	float fr[16];
	float sa;
	int state;
	const char* p = msg;
	char  ipaddr[255];
	vcg::Point4f vel;

	int st = clock();
	sscanf(p,"%s %f %f %f %f  %f %f %f %f  %f %f %f %f  %f %f %f %f %f %f %f %f %f",
		ipaddr,		&fr[0],		&fr[1],		&fr[2],		&fr[3],		&fr[4],		&fr[5],		&fr[6],		&fr[7],		&fr[8],
		&fr[9],		&fr[10],		&fr[11],		&fr[12],		&fr[13],		&fr[14],		&fr[15],		&vel[0],		&vel[1],
		&vel[2],		&vel[3]	, &sa, &state	);


	Race::PlayersIterator pi;
	race.mutex_on_players.enter();

	pi = race.FindPlayer(std::string(ipaddr));
	if(pi == race.players.end()){/* it's a new player*/
 			pi = race.AddPlayer();
			(*pi).ip_address = ipaddr;
			(*pi).car.front_wheel_pos = -1;
			(*pi).car.back_wheel_pos =  1;
	}

	Player & pl = (*pi);

	memcpy(&pl.car.frame[0][0],&fr[0],16*sizeof(float));
	memcpy(&pl.car.vel[0],&vel[0],4*sizeof(float));
	pl.car.steering_angle = sa;
	pl.state = state;
	race.mutex_on_players.leave();

	return 0;
}
